`default_nettype none
`timescale 1ns/1ps

module test_quad_gen ();

// テスト対象の出力データ型
typedef logic [9:0] data_t;

// テスト対象の信号
logic   clock;      // クロック入力
logic   reset;      // Active Highの同期リセット入力
logic   out_valid;  // 出力VALID
logic   out_ready;  // 出力READY
data_t  out_data;   // 出力データ

// クロック生成 (10ns周期)
initial begin
        clock = 0;
end 
always #(5) begin
    clock = ~clock;
end

// テスト対象のインスタンス化
quad_gen dut (.*);

/**
 * @summary テスト対象の出力データを受け取る。
 * @param[out] data テスト対象から出力されたデータ
 * @param[in] ready_deassert_factor READYをデアサートする確率。intの範囲 (0~2^32-1) 大きいほどデアサートしやすくなる。
 **/
task automatic get_output_data(output data_t data, input int ready_deassert_factor);
    bit ready_value;
    do begin
        ready_value = $urandom() >= ready_deassert_factor;
        out_ready <= ready_value;
        @(posedge clock);
    end while( !ready_value );
    while(out_valid == 0) @(posedge clock);
    data = out_data;
    out_ready <= ready_value;
endtask

initial begin
    $dumpfile("test-quad_gen.vcd"); // VCDファイルを出力する
    $dumpvars(0);                    // 全変数をVCDに出力する
    
    // リセットをアサートして戻す
    reset <= 1;
    repeat(2) @(posedge clock);
    reset <= 0;
    @(posedge clock);
    while(!out_valid) @(posedge clock); // 最初のout_validアサートまで待つ

    fork
        begin
            for(int i = 0; i < 1024; i++) begin
                bit expected_valid;
                data_t actual_data;
                expected_valid = (i & 3) == 0;
                if( expected_valid ) begin
                    // 出力データ読み出し (1/2の確率でout_readyデアサート)
                    get_output_data(actual_data, 32'h80000000);
                    // 期待値チェック
                    if( actual_data !== i ) $error("#%04d data mismatch. expected: %04d, actual: %04d", i, i, out_data);
                end
            end
        end
        begin
            // タイムアウト検出。2048サイクル以内でテストが完了することを期待する。
            static int limit = 2048;
            for(int cycles = 0; cycles < limit; cycles++ ) @(posedge clock);
            $error("Error: simulation timed out after %d cycles.", limit);
        end
    join_any
    disable fork;
    $finish;
end

endmodule

`default_nettype wire